package com.itsclicking.clickapp.fluttersocketio;

import android.util.Log;

import com.github.nkzawa.emitter.Emitter;
import com.github.nkzawa.socketio.client.IO;
import com.github.nkzawa.socketio.client.Socket;

import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.Map;

import io.flutter.plugin.common.MethodChannel;

public class SocketIO implements ISocketIO {

    private static final String TAG = "SocketIO";
    private MethodChannel mChannel;
    private String mSocketNameSpace;
    private String mStatusCallback;
    private Socket mSocket;
    private IO.Options mOptions;
    private Map<String, String> mSubscribes;

    public SocketIO(MethodChannel channel, String socketNameSpace, String statusCallback) {
        this.mChannel = channel;
        this.mSocketNameSpace = socketNameSpace;
        this.mStatusCallback = statusCallback;
        mSubscribes = new HashMap<>();
    }

    private boolean isExistedChannel(String channel) {
        if(mSubscribes != null && mSubscribes.size() > 0) {
            return mSubscribes.containsKey(channel);
        }
        return false;
    }

    private void addChannel(Map<String, String> channels) {
        if(channels != null && channels.size() > 0) {
            for(Map.Entry<String, String> channel : channels.entrySet()) {
                addChannel(channel.getKey(), channel.getValue());
            }
        }
    }

    private void addChannel(String channel, String callback) {
        if(mSubscribes == null) {
            mSubscribes = new HashMap<>();
        }
        if(!isExistedChannel(channel)) {
            mSubscribes.put(channel, callback);
        }
    }

    private void removeChannel(Map<String, String> channels) {
        if(channels != null && channels.size() > 0) {
            for(Map.Entry<String, String> channel : channels.entrySet()) {
                removeChannel(channel.getKey());
            }
        }
    }

    private void removeChannel(String channel) {
        if(mSubscribes != null && isExistedChannel(channel)) {
            mSubscribes.remove(channel);
        }
    }

    private void removeChannelAll() {
        if(mSubscribes != null) {
            mSubscribes.clear();
        }
    }

    @Override
    public String getNameSpace() {
        return mSocketNameSpace;
    }

    @Override
    public void connect() {
        try {
            mSocket = (mOptions != null ? IO.socket(mSocketNameSpace, mOptions) : IO.socket(mSocketNameSpace));  //in nodejs you may say that is localhost:3000, 127.0.0.1:3000...etc
            mSocket.on(Socket.EVENT_CONNECT_TIMEOUT, new Emitter.Listener() {
                @Override
                public void call(Object... args) {
                    if (!Utils.isNullOrEmpty(mStatusCallback)) {
                        mChannel.invokeMethod(mStatusCallback, args[0].toString());
                    }
                }
            });

            mSocket.on("socket_info", new Emitter.Listener() {
                @Override
                public void call(Object... args) {
                    Log.d(TAG, "connect success");
                    if (!Utils.isNullOrEmpty(mStatusCallback)) {
                        mChannel.invokeMethod(mStatusCallback, args[0].toString());
                    }
                }
            });
            mSocket.connect();

        } catch (URISyntaxException e) {
            Log.d(TAG, "connect fail : " + e.toString());
            if (!Utils.isNullOrEmpty(mStatusCallback)) {
                mChannel.invokeMethod(mStatusCallback, "failed");
            }
        }
    }

    @Override
    public void sendMessage(String event, String message) {
        if (isConnected() && !Utils.isNullOrEmpty(event)
                && !Utils.isNullOrEmpty(message)) {
            mSocket.emit(event, message);
        }
    }

    @Override
    public void subscribes(String channelName, final String callback) {
        addChannel(channelName, callback);
        mSocket.on(channelName, new Emitter.Listener() {
            @Override
            public void call(Object... args) {
                if (!Utils.isNullOrEmpty(callback)) {
                    mChannel.invokeMethod(callback, args[0].toString());
                }
            }
        });
    }

    @Override
    public void subscribes(Map<String, String> subscribes) {
        if (isConnected() && mChannel != null) {
            if (subscribes != null && subscribes.size() > 0) {
                for (Map.Entry<String, String> sub : subscribes.entrySet()) {
                    if (!Utils.isNullOrEmpty(sub.getKey())) {
                        final String subCallback = sub.getValue();
                        subscribes(sub.getKey(), subCallback);
                    }
                }
            }
        }
    }

    @Override
    public void unSubscribes(String channelName, final String callback) {
        removeChannel(channelName);
        mSocket.off(channelName, new Emitter.Listener() {
            @Override
            public void call(Object... args) {
                if (!Utils.isNullOrEmpty(callback)) {
                    mChannel.invokeMethod(callback, args[0].toString());
                }
            }
        });
    }

    @Override
    public void unSubscribes(Map<String, String> subscribes) {
        if (isConnected() && mChannel != null) {
            if (subscribes != null && subscribes.size() > 0) {
                for (Map.Entry<String, String> sub : subscribes.entrySet()) {
                    if (!Utils.isNullOrEmpty(sub.getKey())) {
                        final String subCallback = sub.getValue();
                        unSubscribes(sub.getKey(), subCallback);
                    }
                }
            }
        }
    }

    @Override
    public void unSubscribesAll() {
        if(mSubscribes != null && mSubscribes.size() > 0) {
            unSubscribes(mSubscribes);
        }
    }

    @Override
    public void disconnect() {
        if (mSocket != null) {
            mSocket.disconnect();
        }
    }

    @Override
    public void destroy() {
        disconnect();
        unSubscribesAll();
        mSocket = null;
        mSocketNameSpace = null;
        mStatusCallback = null;
        mOptions = null;
        mChannel = null;
        mSubscribes = null;
    }

    @Override
    public boolean isConnected() {
        return mSocket != null && mSocket.connected();
    }
}
